
#
# BioPerl module for Bio::Pfam::Bait::BaitOutput
#
# Cared for by Kevin Howe <pfam@sanger.ac.uk>
#
# Copyright Kevin Howe
#
# You may distribute this module under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

BaitOutput - DESCRIPTION of Object

=head1 SYNOPSIS

Give standard usage here

=head1 DESCRIPTION

Describe the object here

=head1 CONTACT

Mail pfam@sanger.ac.uk with any queries

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut


# Let the code begin...


package Bio::Pfam::Bait::BaitOutput;
use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use strict;

# Object preamble - inherits from Bio::Root::Object

use Bio::Root::Object;


@ISA = qw(Bio::Root::Object Exporter);
# new() is inherited from Bio::Root::Object

# _initialize is where the heavy stuff will happen when new is called

sub _initialize {
  my($self, $annSeq) = @_;

  my $make = $self->SUPER::_initialize;

  $self->{'baitOutput_seq'} = $annSeq->sequence();
  my @array =  Bio::Pfam::Bait::BaitOutput->_getAnnotationRecords( $annSeq );
  $self->{'baitOutput_records'} = \@array;

  return $make; # success - we hope!
}



=head2 write

 Title   : write
 Usage   : $baitOut->write(\*HANDLE);
 Function: Writes the annotation in standard format to the given handle  
 Returns : 
 Args    : A file handle

=cut

sub write{
   my ($self,$file, $scoresNeeded) = @_;

   my $seqName = $self->{'baitOutput_seq'}->id();
   foreach my $reg (@{$self->{'baitOutput_records'}}) {
       my $extent = $reg->from()."-".$reg->to();
       my $score = "";
       if (defined $reg->scoreOfHighestBait() and $scoresNeeded ) { 
	   $score = "[".$reg->highestBait().",".$reg->scoreOfHighestBait()."]";
       }
       printf $file "%-10s %-10s%-50s %-19s%s\n", $seqName, $extent, 
           $reg->annotation()->description(), $score;
       $seqName = "";
   }
}



=head2 writeFamily

 Title   : writeFamily (CLASS METHOD)
 Usage   : Bio::Pfam::Bait::BaitOutput::writeFamily($fh, $scoresRequired, \@listOfAnnSeq, );
 Function:
    Takes a list of annotated sequences already bait annotated and writes them
    to the given file handle in a standard format
 Returns : 
 Args    : File handle, scores reuired or not

=cut

sub writeFamily {
   my ($package, $file, $scoresNeeded, $annSeqListRef) = @_;

   foreach my $annSeq (@$annSeqListRef) {
        my @refList = $package->_getAnnotationRecords( $annSeq );

	my $seqName = $annSeq->sequence()->id();
	my $swissprotDescription = $annSeq->sequence()->desc();
        # now output the result

        my $scoreField = "%s";
        if ( $scoresNeeded ) {
            $scoreField = "%-19s";
        }

        foreach my $reg (@refList) {
            my $extent = $reg->from()."-".$reg->to();
            my $score = "";
            if ( $scoresNeeded && defined( $reg->scoreOfHighestBait()) )
	    {
                $score = "[".$reg->highestBait().",".$reg->scoreOfHighestBait()."]";
            } 
            printf $file "%-10s %-10s%-50s $scoreField%s\n", $seqName, $extent, 
                $reg->annotation()->description(), $score,  $swissprotDescription;
            $seqName = $swissprotDescription  = "";
        }
    }
}



=head2 writeHTML

 Title   : writeHTML
 Usage   : $baitOut->writeHTML();
 Function:
    The simplified bait annotation now stored in this object is written printed
    to the web-browser
 Returns : 
 Args    :

=cut

sub writeHTML{
   my ($self, $file, $defaultsRequired) = @_;
   my (@default, @bait);
   
   foreach my $reg (@{$self->{'baitOutput_records'}}) {
       if ($reg->type() eq $Bio::Pfam::Bait::BaitRegion::_DEFAULT) {
	   push @default, $reg;
       }
       else {
	   push @bait, $reg;
       }
   }

   if (@bait) {
       print $file "<p><center>More specific annotation for the following regions:\n<p>\n";
       print $file "<table border>\n";
       print $file "<tr><th>Start</th><th>End</th><th>Annotation</th></tr>\n";

       foreach my $el (@bait) {
	   print $file "<tr>";

	   print $file "<td align=center>";
	   print $file $el->from()."<br>";
	   print $file "</td>";

	   print $file "<td align=center>";
	   print $file $el->to()."<br>";
	   print $file "</td>";

	   print $file "<td align=center>";
	   print $file $el->annotation->description()."<br>";
	   print $file "</td>";

	   print "</tr>\n";
       }
       print "</table></center>\n<p>";
   }

   if (@default and $defaultsRequired) {
       print $file "<p><center>Default Pfam annotation for the following regions:\n<p>\n";
  
       print $file "<table border>\n";
       print $file "<tr><th>Start</th><th>End</th><th>Annotation</th></tr>\n";

       foreach my $el (@default) {
	   print $file "<tr>";

	   print $file "<td align=center>";
	   print $file $el->from()."<br>";
	   print $file "</td>";

	   print $file "<td align=center>";
	   print $file $el->to()."<br>";
	   print $file "</td>";

	   print $file "<td align=center>";
	   print $file $el->annotation->description()."<br>";
	   print $file "</td>";

	   print "</tr>\n";
       }

       print "</table></center>\n<p>";
   }
}




=head2 writeFamilyHTML

 Title   : writeFamilyHTML (CLASS METHOD)
 Usage   : Bio::Pfam::Bait::BaitOutput::writeFamilyHTML($file, $scoresRequired, $listOfannSeqRef );
 Function:
    This function is given the list of annotated sequences resulting from the
    bait annotation of a whole family and writes out an html table showing the
    results.
 Returns : 
 Args    : A list of AnnotatedSequence

=cut

sub writeFamilyHTML {
   my ($package, $file, $scoresNeeded, $seqsRef) = @_;

   print $file "<table border>\n";
   print $file "<tr><th>Sequence</th><th>Start</th><th>End</th><th>Best match</th>"
       ."<th>Bait Annotation</th><th>Swissprot Annotation</th></tr>\n";
   foreach my $annSeq (@$seqsRef) {
       my @refList = $package->_getAnnotationRecords( $annSeq );
       
       my $seqName = $annSeq->sequence()->id();
       my $swissprotDescription = $annSeq->sequence()->desc();
       # now output the result
       
       print $file "<tr>";
       print $file "<td><a href=\"$PfamWWWConfig::swisspfam?name=$seqName\">$seqName</a></td>";

       print $file "<td align=center>";
       if (not @refList) {
	   print $file "-";
       }
       foreach my $reg (@refList) {
	   print $file $reg->from()."<br>";
       }
       print $file "</td>";

       print $file "<td align=center>";
       if (not @refList) {
	   print $file "-";
       }
       foreach my $reg (@refList) {
	   print $file $reg->to()."<br>";
       }
       print $file "</td>";

       my $written = 0;
       print $file "<td align=center>";
       foreach my $reg (@refList) {
	   my $score = ""; 
	   if ( defined( $reg->scoreOfHighestBait()) ){
	       $score = "[".$reg->highestBait().",".$reg->scoreOfHighestBait()."]";
	       $written = 1;
	   }
	   print $file "$score<br>";
       }
       if (not $written) {
	   print $file "-";
       }
       print $file "</td>";

       print $file "<td>";
       if (not @refList) {
	   print $file "None";
       }
       foreach my $reg (@refList) {
	   print $file $reg->annotation()->description()."<br>";
       }
       print $file "</td>";

       print $file "<td>";
       print $file $swissprotDescription;
       print $file "</td>";

       print "</tr>\n";
    }
   print "</table>\n<p>";
       
}




=head2 _getAnnotationRecords

 Title   : getAnnotationRecords (CLASS METHOD)
 Usage   : @list =  Bio::Pfam::Bait::BaitOutput::_getBaitAnnotation( $annSeq )
 Function:
    This function takes an AnnotatedSequence previously filled with
    both PFamRegions and BaitRegions by the BaitAnnotation process (i.e.
    resolveAnnotatedSequence has already been called) and makes decisions
    as to what the true annotation for the sequence is. The decision is
    returned in the form of a list of BaitRegion.

    The rules are as follows:
    
    1. If the list of regions contains one or more whole sequence annotations
       that are not default ones, ONLY these regions are included in the return
       list.
    2. IF the list of regions contains one or more sequence annotations, all
       of which are default, and there are no domain-only annotated that
       do not contain default annotations, then ONLY these regions are included in 
       the return list.
    3. If there are no sequence regions with non-default annotations and there
       is one or more domain-only regions with non-default annotations, then
       the annotation reverts to a domain one, and ONLY domain annotated regions 
       are returned.

 Args    : 
    1. A ref. to AnnotatedSequence, previosly annotated by resolveAnnotatedSequence.
 Returns:
    A list of AnnotatedRegion
 Note:
    This function assumes that if a domain occurs twice or more on a sequence,
    then EITHER all subsequences in the domain have been annotated individually
    (domain search-type appeared in the bait file) or that one of the domains
    was annotated and a _REPEAT record was added to the list for the other
    occurrences (sequence search-type appeared in the bait file), i.e. that
    there is no mixing of search types across a single family. This assumption
    always holds if the wider "one family - one bait file" assumption holds.

=cut

sub _getAnnotationRecords{
   my ($package, $annSeq, $baitList, $defaultList) = @_;
   my (%regionHash, %extraOccurrences, @baitRegionList );

   # firstly, construct two lists, one each for Pfam regions and Bait regions resp.
   # shouldn't need to do any other introspection other than this (nasty I know, but
   # it is the least of many inelegant evils)

   my (@pfamRegs, @baitRegs);
   foreach my $reg ($annSeq->eachAnnotatedRegion() ) {
       if ( ref($reg) eq "Bio::Pfam::PfamRegion" ) {
	   push @pfamRegs, $reg;
       }
       elsif ( ref( $reg) eq "Bio::Pfam::Bait::BaitRegion" ) {
	   push @baitRegs, $reg; 
       }
   }

   while ( @baitRegs ) {
       my $currentPfamReg = shift @pfamRegs;
       my $currentSfrReg = shift @baitRegs;

       if ( $currentSfrReg->type() eq $Bio::Pfam::Bait::BaitRegion::_DELEGATE ) {
	   next;
       }
       elsif ( $currentSfrReg->type() eq $Bio::Pfam::Bait::BaitRegion::_BAIT ) {
	       push @baitRegionList, $currentSfrReg;	   
	       if (defined $currentPfamReg) {
		   $regionHash{ $currentPfamReg->accession() } = $currentSfrReg;
	       }
       }
       elsif ( $currentSfrReg->type() eq $Bio::Pfam::Bait::BaitRegion::_DEFAULT ) {
	   $currentSfrReg->annotation( $currentPfamReg->annotation() );
	   push @baitRegionList, $currentSfrReg;	   

	   # yes it may be the case that the following actually overwrites an an earlier
	   # annotation for the same domain. If that is the case, then we are annotating 
	   # that family on a domain basis, so will never have cause to look at that entry 
	   # in the hash (because no _REPEAT records for that family will subsequently appear

	   $regionHash{ $currentPfamReg->accession() } = $currentSfrReg;
       }
       elsif ( $currentSfrReg->type() eq $Bio::Pfam::Bait::BaitRegion::_REPEAT ) {
	   $extraOccurrences{ $currentPfamReg->accession() }++;

	   # This will change the annotation in the baitList, since the hash just contains references
	   # to BaitRegions
       }
   }
   
   # Next, for repeat domains, we have to append an "occurrences" tag to the annotation
   foreach my $item (keys %regionHash) {
       if ($extraOccurrences{ $item } ) {
	   $regionHash{ $item }->annotation()->description(
	       $regionHash{ $item }->annotation()->description()
	       ." (".($extraOccurrences{ $item }+1)."copies)");
       }
   }


   # Right. Now we have a list of BaitRegions, but we are not finished yet. We have to go through and 
   # check which correspond to domain annotations and which to sequence annotations.

   my ($delete, @sequenceList, @domainList, %deleteList);
   foreach my $reg ( @baitRegionList ) {
       if ( $reg->from() eq $annSeq->sequence()->start() and 
	        $reg->to() eq $annSeq->sequence()->end() ) {
	   push @sequenceList, $reg;
       }
       else {
	   push @domainList, $reg;
       }
   }
   # we now have two list, one for the default and one for the bait region list. Now we have to post process

   $delete = 0;
   foreach my $reg ( @sequenceList ) {
       if ($reg->type() eq $Bio::Pfam::Bait::BaitRegion::_DEFAULT) {
	   $deleteList{$reg} = 1;
       }
       else {
	   $delete = 1;
       }
   }
   if ($delete) {
       @sequenceList = grep { not ( defined($deleteList{$_}) ) } @sequenceList;
       @domainList = ();
   }

   $delete = 1;
   foreach my $reg ( @domainList ) {
       # if we get here, then we must have to process the list for bait annotations. If there are some,
       # then we can return just the domain annotations. If none, then we return just the sequence annotations
       if ($reg->type() eq $Bio::Pfam::Bait::BaitRegion::_BAIT) {
	   @sequenceList = ();
	   $delete = 0;
	   last;
       }
   }
   
   if ($delete and @sequenceList) {
       # then ther must have been no non-default domain annotations. Just return the sequence ones
       # we get here if either there were no domain annotations (in which case the following is harmless)
       # or if both lists are filled with only default annotations, in which case we bowto the 
       # sequence annotations

       @domainList = ();
   }
       
   return (@sequenceList, @domainList);
}




