
package Align;

use Sequence;
use SequenceSet;
use Carp;

my %fields = (
	      name  => undef,
	      seq => undef   # array of sequence objects
);

sub new {
    my $ref = shift;
    my $class = ref($ref) || $ref;
    
    my $self = {
	_permitted => \%fields,
	%fields, };

    # seq is an array of objects

    $self->{seq} = [];

    bless $self, $class;
    return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $type = ref($self) || warn "Ok, $self is not an object of mine!";
    my $name = $AUTOLOAD;
    $name =~ s/.*://; #get only the bit we want
    unless (exists $self->{_permitted}->{$name} ) {
	warn "In type $type, can't access $name $keys  ";
    }
    if (@_) {
	return $self->{$name} = shift;
    } else {
	return $self->{$name}
    }
}

sub DESTROY {
}

sub read_fasta {
    my $self = shift;
    my $in = shift;
    my $count = 0;
    my $start,$end,$name,$seqname,$seq,$seqchar;

    while( <$in> ) {
	if( /^>(\S+)/ ) {
	    $tempname = $1;
	    if( defined $name ) {
		# put away last name and sequence 

		$seq = new Sequence;

		if( $name =~ /(\S+)\/(\d+)-(\d+)/ ) {
		    $seqname = $1;
		    $start = $2;
		    $end = $3;
		} else {
		    $seqname=$name;
		    $start = 1;
		    $end = length($align{$name});
		}
		

		$seq->name($seqname);
		$seq->seq($seqchar);
		$seq->start($start);
		$seq->end($end);

		push(@{$self->{seq}},$seq);
		$count++;
	    }
	    $name = $tempname;
	    $seqchar  = "";
	    next;
	}
	s/[^A-Za-z\.\-]//g;
	$seqchar .= $_;

    }
    # put away last name and sequence 
    
    $seq = new Sequence;
    
    if( $name =~ /(\S+)\/(\d+)-(\d+)/ ) {
	$seqname = $1;
	$start = $2;
	$end = $3;
    } else {
	$seqname=$name;
	$start = 1;
	$end = length($align{$name});
    }
    
    
    $seq->name($seqname);
    $seq->seq($seqchar);
    $seq->start($start);
    $seq->end($end);
    
    push(@{$self->{seq}},$seq);
    $count++;

    return $count;
}
	    

sub read_selex {
    my $self = shift;
    my $in = shift;
    my $start,$end,%align,$name,$seqname;
    
    # not quite sure 'what' selex format is, but here we go!

    while( <$in> ) {
	/^#/ && next;
	/^\s/ && next;
	last;
    }

    # first line starting with word
    chop;

    if( !/^(\S+)\s+([A-Za-z\.\-]+)\s*/ ) {
	warn("Ok. I think [$_] does not look like selex format. Not sure...");
	return (-1);
    }

    $name = $1;
    $seq = $2;

    $align{$name} .= $seq;

    while( <$in> ) {
	!/^([^\#]\S+)\s+([A-Za-z\.\-]+)\s*/ && next;
	
	$name = $1;
	$seq = $2;

	$align{$name} .= $seq;
    }

    # ok... now we can make the sequences

    $count = 0;
    foreach $name ( keys %align ) {
	$seq = new Sequence;

	if( $name =~ /(\S+)\/(\d+)-(\d+)/ ) {
	    $seqname = $1;
	    $start = $2;
	    $end = $3;
	} else {
	    $seqname=$name;
	    $start = 1;
	    $end = length($align{$name});
	}


	$seq->name($seqname);
	$seq->seq($align{$name});
	$seq->start($start);
	$seq->end($end);

	push(@{$self->{seq}},$seq);
	$count++;
    }

    return $count;
}
	
sub read_mase {
    my $self = shift;
    my $in = shift;
    my $name;
    my $start;
    my $end;
    my $seq;
    my $add;
    my $count = 0;
	
	
    while( <$in> ) {
	/^;/ && next;
	if( /^(\S+)\/(\d+)-(\d+)/ ) {
	    $name = $1;
	    $start = $2;
	    $end = $3;
	} else {
	    $name = split(/\s/);
	    $end = -1;
	}

	$seq = "";

	while( <$in> ) {
	    /^;/ && last;
	    s/[^A-Za-z\.\-]//g;
	    $seq .= $_;
	}

	$add = new Sequence;
	$add->name($name);
	$add->seq($seq);
	$add->start($start);
	$add->end($end);

	if( $end == -1) {
	    $start = 1;
	    $end = length $seq;
	}

	push(@{$self->{seq}},$add);
	$count++;
    }

    return $count;
}
	
	
sub read_Pfam {
    my $self = shift;
    my $in = shift;
    my $name;
    my $start;
    my $end;
    my $seq;
    my $add;
    my $count = 0;

    while( <$in> ) {
	chop;
	if( !/^(\S+)\/(\d+)\-(\d+)\s+(\S+)\s*$/ ) { 
	    carp "Found a bad line [$_] in the pfam format alignment";
	    next;
	} 
	$name = $1;
	$start = $2;
	$end = $3;
	$seq = $4;



	$add = new Sequence;
	$add->name($name);
	$add->seq($seq);
	$add->start($start);
	$add->end($end);

	push(@{$self->{seq}},$add);
	$count++;
    }

    return $count;
}

sub write_Pfam {
    my $self = shift;
    my $out  = shift;

    foreach $seq ( $self->eachSequence() ) {
	$namestr = sprintf("%s/%d-%d",$seq->name(),$seq->start(),$seq->end());
	print $out sprintf("%-20s  %s\n",$namestr,$seq->seq());
    }
}

sub write_fasta {
    my $self = shift;
    my $out  = shift;
    my $dostart = shift;

    foreach $seq ( $self->eachSequence() ) {
	$seq->write_fasta($out,$dostart);
    }
}
    

sub eachSequence {
    my $self = shift;
    my @arr;

    @arr = @{$self->{seq}};

    return @arr;
}

sub show_Pfam {
    my $self = shift;
    my $out  = shift;

    foreach $seq ( $self->eachSequence() ) {
	print $out sprintf("%s/%d-%d %s\n",$seq->name(),$seq->start(),$seq->end(),$seq->seq());
    }

}

    


