
#
# BioPerl module for Bio::Pfam
#
# Cared for by Ewan Birney <birney@sanger.ac.uk>
#
# Copyright Ewan Birney
#
# You may distribute this module under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

Bio::Pfam - Object factory that provides Pfam based objects

=head1 SYNOPSIS

    $db = Bio::Pfam::default_db(); # gets a Bio::Pfam::DB object from
                                   # defualts

=head1 DESCRIPTION

This is really a package, not an object, providing the defaults for a system
for the Pfam databases. By using only the functionality in this and the
Bio::Pfam::DB module you should be insulated from the changes of the database
layout.

This module allows you to instantiate database objects. It is here where the
defaults of the database system are held. You should have a look at 
Bio::Pfam::DB for what methods you are allowed to use.

=head1 CONTACT

Describe contact details here

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut


# Let the code begin...


package Bio::Pfam;
use vars qw($AUTOLOAD @ISA);
use strict;
no strict 'vars';

use Bio::Pfam::DB;
use Bio::Pfam::DB_RCS;
use Bio::Pfam::DB_Web;

$pfam_root_dir   = '/nfs/disk100/pubseq/Pfam';
#$pfam_root_dir  = '/nfs/disk21/birney/prog/pfam';

$rcs_current_dir = "$pfam_root_dir/CURRENT";
$rcs_master_dir  = "$pfam_root_dir/RCS_MASTER";
$rcs_attic_dir   = "$pfam_root_dir/RCS_ATTIC";

$rcs_index_file  = "$pfam_root_dir/ACCESSION/accmap.dat";
$rcs_lock_file   = "$pfam_root_dir/ACCESSION/accmap.lock";
$rcs_srs_db      = 'pfamseq';
$fasta_db        = '/nfs/disk100/pubseq/pfamseq/pfamseq';

$web_srs_pfamseq = 'pfamseq';
$web_srs_pfam = 'pfam';
$web_data_root = '/nfs/WWW/htdocs/Software/Pfam/data/';

=head2 default_db

 Title   : default_db
 Usage   : $db = &Bio::Pfam::default_db();
 Function: returns the default Pfam::DB object for this system
           This could be an RCS db or a flat-file based db -
           you shouldnt care
 Example :
 Returns : new Bio::Pfam::DB object (probably something that inheriets it in fact).
 Args    :


=cut

sub default_db{
    return Bio::Pfam::DB_RCS->new(-current => $rcs_current_dir, '-index' => $rcs_index_file, -srsdb => $rcs_srs_db, -lock_file => $rcs_lock_file , '-fastadb' => $fasta_db);
}


=head2 latest_release

 Title   : latest_release
 Usage   : $db = &Bio::Pfam::latest_release();
 Function: returns the latest fixed release of Pfam
           as Pfam::DB object
 Returns : a new Bio::Pfam::DB object (probably something that inheriets it in fact)
 Args    :

=cut

sub latest_release{
   my ($self) = @_;

    return Bio::Pfam::DB_Web->new('-pfamseqdb' => $web_srs_pfamseq, 
				  '-pfamdb' => $web_srs_pfam,
				  '-datadir' => $web_data_root);
}

