
#
# BioPerl module for Bio::Pfam::Entry_RCS
#
# Cared for by Ewan Birney <pfam@sanger.ac.uk>
#
# Copyright Ewan Birney
#
# You may distribute this module under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

Bio::Pfam::Entry_RCS - DESCRIPTION of Object

=head1 SYNOPSIS

Give standard usage here

=head1 DESCRIPTION

Describe the object here

=head1 CONTACT

Describe contact details here

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut


# Let the code begin...


package Bio::Pfam::Entry_RCS;
use vars qw($AUTOLOAD @ISA);
use strict;

# Object preamble - inheriets from Bio::Root::Object

use Bio::Root::Object;
use Bio::Pfam::Entry;

@ISA = qw(Bio::Pfam::Entry Exporter);

# new() is inherited from Bio::Root::Object

# _initialize is where the heavy stuff will happen when new is called

sub _initialize {
  my($self,@args) = @_;

  my $make = $self->SUPER::_initialize;

# set stuff in self from @args
 return $make; # success - we hope!
}


=head2 seed

 Title   : seed
 Usage   : $seed = $self->seed()
 Function:
 Example :
 Returns : Bio::SimpleAlign object of the seed alignment
 Args    :


=cut

sub seed{
   my ($self,@args) = @_;
   my($dir,$id,$out);
   $dir = $self->_directory();
   $id  = $self->id();

   open(_ALIGN,"$dir/SEED") || $self->throw("For entry object [$id], got no valid directory for SEED alignment [$dir/SEED] $!");
   
   $out = Bio::SimpleAlign->new();
   $out->read_Pfam(\*_ALIGN);

   close(_ALIGN);

   return $out;
					    
}


=head2 full

 Title   : full
 Usage   : $full = $self->full()
 Function:
 Example :
 Returns : Bio::SimpleAlign object of the full alignment
 Args    :


=cut

sub full{
   my ($self,@args) = @_;
   my($dir,$id,$out);
   $dir = $self->_directory();
   $id  = $self->id();

   open(_ALIGN,"$dir/ALIGN") || $self->throw("For entry object [$id], got no valid directory for ALIGN alignment [$dir/SEED] $!");
   
   $out = Bio::SimpleAlign->new();
   $out->read_Pfam(\*_ALIGN);

   close(_ALIGN);

   return $out;
}


=head2 name_start_end

 Title   : 
 Usage   : @names = $self->name_start_end('FULL')
 Function:
 Example :
 Returns : Bio::SimpleAlign object of the full alignment
 Args    :


=cut


=head2 ann

 Title   : ann
 Usage   : $self->ann($newval)
 Function: 
 Example : 
 Returns : Annotation object
 Args    : 


=cut

sub ann{
   my ($self,$value) = @_;
   my($dir,$id,$out);
   $dir = $self->_directory();
   $id  = $self->id();

   open(_DESC,"$dir/DESC") || $self->throw("For entry object [$id], got no valid directory for DESC [$dir/DESC] $!");
   
   $out = Bio::Pfam::Annotation->new();
   $self->_read_std_desc(\*_DESC,$out);

   close(_DESC) || $self->throw("Could not close DESC");

   return $out;
     
}


sub name_start_end {
   my ($self,$type) = @_;

   my($dir,$id,$out,$file,@list,$name);

   $dir = $self->_directory();

   $id  = $self->id();

   SWITCH : { 
     $type =~ /FULL/ && do { $file = 'ALIGN'; last; };
     $type =~ /SEED/ && do { $file = 'SEED'; last; };
     $self->throw("This type [$type] is neither full nor seed. Can't process");
   }
 
   open(_ALIGN,"$dir/$file") || $self->throw("For entry object [$id], got no valid directory for alignment [$dir/$file] $!");
   
   while(<_ALIGN>) {
     ($name) = split;
     push(@list,$name);
   }

   close(_ALIGN) || $self->throw("Could not close alignment file handle");

   return @list;
}


=head2 _directory

 Title   : _directory
 Usage   : $obj->_directory($newval)
 Function: Internal directory for the entry
 Example : 
 Returns : value of _directory
 Args    : newvalue (optional)


=cut

sub _directory{
   my ($self,$value) = @_;
   if( defined $value) {
      $self->{'_directory'} = $value;
    }
    return $self->{'_directory'};

}

