
#
# BioPerl module for Bio::Pfam::Entry_Web
#
# Cared for by Ewan Birney <pfam@sanger.ac.uk>
#
# Copyright Ewan Birney
#
# You may distribute this module under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

Bio::Pfam::Entry_Web - DESCRIPTION of Object

=head1 SYNOPSIS

Give standard usage here

=head1 DESCRIPTION

Describe the object here

=head1 CONTACT

Describe contact details here

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut


# Let the code begin...


package Bio::Pfam::Entry_Web;
use vars qw($AUTOLOAD @ISA);
use strict;

# Object preamble - inheriets from Bio::Root::Object

use Bio::Root::Object;
use Bio::Pfam::Entry;

@ISA = qw(Bio::Pfam::Entry Exporter);

# new() is inherited from Bio::Root::Object

# _initialize is where the heavy stuff will happen when new is called

sub _initialize {
  my($self,@args) = @_;

  my $make = $self->SUPER::_initialize;

# set stuff in self from @args
 return $make; # success - we hope!
}


=head2 seed

 Title   : seed
 Usage   : $seed = $self->seed()
 Function:
 Example :
 Returns : Bio::SimpleAlign object of the seed alignment
 Args    :


=cut

sub seed{
   my ($self,@args) = @_;
   my($dir,$id,$out,$acc);
   $dir = $self->_datadir();
   $acc  = $self->acc();

   open(_ALIGN,"gunzip -c $dir/seed/$acc.full.gz |") || $self->throw("For entry object [$id], got no valid directory for SEED alignment [$dir/SEED] $!");
   
   $out = Bio::SimpleAlign->new();
   $out->read_Pfam(\*_ALIGN);

   close(_ALIGN);

   return $out;
					    
}


=head2 full

 Title   : full
 Usage   : $full = $self->full()
 Function:
 Example :
 Returns : Bio::SimpleAlign object of the full alignment
 Args    :


=cut

sub full{
   my ($self,@args) = @_;
   my($dir,$id,$out,$acc);

   $dir = $self->_datadir();
   $acc  = $self->acc();

   open(_ALIGN,"gunzip -c $dir/full/$acc.full.gz |") || $self->throw("For entry object [$id], got no valid directory for ALIGN alignment [$dir/SEED] $!");
   
   $out = Bio::SimpleAlign->new();
   $out->read_Pfam(\*_ALIGN);

   close(_ALIGN);

   return $out;
}



=head2 ann

 Title   : ann
 Usage   : $self->ann($newval)
 Function: 
 Example : 
 Returns : Annotation object
 Args    : 


=cut

sub ann{
   my ($self,$value) = @_;
   my($dir,$id,$out,$acc);

   $dir = $self->_datadir();
   $acc  = $self->acc();

   open(_DESC,"$dir/desc/$acc.desc") || $self->throw("For entry object [$id], got no valid directory for desc file [$dir/desc/$acc.desc] $!");

   $out = Bio::Pfam::Annotation->new();
   $self->_read_std_desc(\*_DESC,$out);

   close(_DESC) || $self->throw("Could not close DESC $!");

   return $out;
     
}

=head2 name_start_end

 Title   : 
 Usage   : @names = $self->name_start_end('FULL')
 Function:
 Example :
 Returns : a list of "name/start-end" for the alignment of choice 
 Args    :


=cut

sub name_start_end {
   my ($self,$type) = @_;

   my($dir,$id,$out,$file,$name, $acc, @list);

   $dir = $self->_datadir();
   $acc  = $self->acc();


   SWITCH : { 
     $type =~ /FULL/ && do { $file = 'full'; last; };
     $type =~ /SEED/ && do { $file = 'seed'; last; };
     $self->throw("This type [$type] is neither full nor seed. Can't process");
   }
 

   open(_ALIGN,"gunzip -c $dir/$file/$acc.full.gz |") || $self->throw("For entry object [$id], got no valid directory for ALIGN alignment [$dir/SEED] $!");
   
   while(<_ALIGN>) {
     ($name) = split;
     push(@list,$name);
   }

   close(_ALIGN) || $self->throw("Could not close alignment file handle (pipe for gunzip)");

   return @list;
}


=head2 _datadir

 Title   : _datadir
 Usage   : $obj->_directory($newval)
 Function: Internal data directory for the entry
 Example : 
 Returns : value of _datadir
 Args    : newvalue (optional)


=cut

sub _datadir{
   my ($self,$value) = @_;
   if( defined $value) {
      $self->{'_datadir'} = $value;
    }
    return $self->{'_datadir'};

}
