

package PfamModel;

use Carp;
use Align;
use PfamDatabase;
# use strict;

use vars qw($AUTOLOAD);

#@EXPORT = qw(&fromstring  &from_hmmls_string);


my %fields = ( 
	      name  => undef,
	      hmmfile => undef,
	      hmmprog => "hmmls",
	      thresh => 15,
	      random => 1,
	      rndfile => "",
	      align => undef,
	      seed  => undef
	      );


$release_dir = "/nfs/disk17/esr/Pfam/RELEASE_2.0";
$current_dir = "/nfs/disk100/pubseq/Pfam/CURRENT"; # RCS 'current' root


sub compare_overlap_to_current_ends {
    my $one = shift;
    my $fam_ref = shift;
    my (%hash,$left_ext,$right_ext,$left_name,$right_name);

    @families = @{$fam_ref};

    foreach $seq ( $one->align()->eachSequence() ) {
	$hash{$seq->name()} .= sprintf("%d-%d:",$seq->start(),$seq->end());
	push(@names,$seq->name());
    }

    #
    # foreach family, foreach sequence, find if it is in other sequence.
    #

    $left_ext =  100000;
    $right_ext = 100000;
    
    FAMILY :
    foreach $family ( @families ) {
	
	if( ! -e "$current_dir/$family/ALIGN" ) {
	    if( defined $report ) { 
		carp("$family has no Align. Skipping on qc");
	    }
	    next;
	}
	
	$model = new PfamModel;
	$model->get_current_model($family);
	$count = 0;
	foreach $seq ( $model->align()->eachSequence() ) {
	    if( $hash{$seq->name()} ) {
		$_ = $hash{$seq->name()};
		chop; # trailing :
		@startstop = split(/:/);
		foreach $startstop ( @startstop) {
		    ($start,$stop) = split(/-/,$startstop);
		    if( ($start >= $seq->start() && $stop <= $seq->end()) || 
		        ($start <= $seq->end() && $stop >= $seq->end()) ||  
		        ($start <= $seq->start && $stop >= $seq->start()) ) {
			push(@arry,sprintf("%s:%s-%d-%d:%s-%d-%d",$seq->name(),$one->name(),$start,$stop,$model->name,$seq->start(),$seq->end()));
			$count++;
		    } else {

			# other domain is either to the left or to the right.

			if( $start > $seq->end() ) {
			    # to the left;
			    
			    $diff = ($start - $seq->end()-1);
			    if( $diff < $left_ext ) {
				$left_ext = $diff;
				$left_name = $seq->name();
			    }
			} elsif ( $stop < $seq->start() ) {
			    # to the right;

			    $diff = ($seq->start() - $stop - 1);
			    if( $diff < $right_ext ) {
				$right_ext = $diff;
				$right_name = $seq->name();
			    }
			} else {
			    carp("Got a non overlapping domain but it is neither left nor right!");
			}
		    }
				

		}
	    }
	}
	if( $report ) {
	    print $report sprintf("Done Model %-25s - Found %d overlaps\n",$model->name(),$count);
	}

	$model = undef; # free Model
    } # end of for each family

    $rethash = {};
    $rethash->{'array'} = \@arry;
    $rethash->{'extleft'} = $left_ext;
    $rethash->{'extright'} = $right_ext;
    $rethash->{'leftname'} = $left_name;
    $rethash->{'rightname'} = $right_name;

    return $rethash;
}

sub show {
    my $self = shift;
    my $file = shift;

    if( !$file) {
	$file = \*STDOUT;
    }
    
    print $file sprintf("Model %s [%s %s] %d\n",$self->name(),$self->hmmfile(),$self->hmmprog(),$self->thresh());
    if( $self->seed() ) {
	print $file "SEED alignment\n";
	$self->seed()->show_Pfam($file);
    }
    if( $self->align() ) {
	print $file "FULL alignment\n";
	$self->align()->show_Pfam($file);
    }

}
    
    
sub compare_overlap_to_current {
    my $one = shift;
    my $fam_ref = shift;
    my $report = shift;
    my @families;
    my @arry;
    my %hash;
    my $count=0;

    # use main ALIGN-ment.

    # build a hash of one on sequence names:

    foreach $seq ( $one->align()->eachSequence() ) {
	$hash{$seq->name()} .= sprintf("%d-%d:",$seq->start(),$seq->end());
	push(@names,$seq->name());
    }

    foreach $seq ( $one->seed()->eachSequence() ) {
	$hash{$seq->name()} .= sprintf("%d-%d:",$seq->start(),$seq->end());
	push(@names,$seq->name());
    }

    #
    # get current families to loop over: from the reference
    #

    @families = @{$fam_ref};

    #
    # foreach family, foreach sequence, find if it is in other sequence.
    #

    FAMILY :
    foreach $family ( @families ) {
	
	if( ! -e "$current_dir/$family/ALIGN" ) {
	    if( defined $report ) { 
		carp("$family has no Align. Skipping on qc");
	    }
	    next;
	}

	#
	# first - open ALIGN, loop through on a regex on the names
	#

	

	$seen = 0;
	$count = 0;

	open(ALIGN,"$current_dir/$family/scores");
	while(<ALIGN>) {
	    /(\S+)\// || next;
	    $test = $1;
	    if( exists $hash{$test} ) {
		$seen = 1;
		last;
	    }
	}
	close(ALIGN);

	if( ! -e "$current_dir/$family/SEED" ) {
	    carp("$family has no seed. Skipping on qc");
	    next;
	}

	open(SEED,"$current_dir/$family/SEED");
	while(<SEED>) {
	    /(\S+)\// || next;
	    $test = $1;
	    if( exists $hash{$test} ) {
		$seen = 1;
		last;
	    }
	}
	close(SEED);

	if( $seen == 0 ) {
	    if( $report ) {
		print $report sprintf("Done Model %-25s - Found %d overlaps\n",$family,$count);
	    }
	    next FAMILY;
	}

	
	$model = new PfamModel;
	$model->get_current_model($family);
	$count = 0;
	@list = $model->align()->eachSequence();
	push(@list,$model->seed()->eachSequence());

	foreach $seq ( @list ) {
	    if( $hash{$seq->name()} ) {
		$_ = $hash{$seq->name()};
		chop; # trailing :
		@startstop = split(/:/);
		foreach $startstop ( @startstop) {
		    ($start,$stop) = split(/-/,$startstop);
		    if( ($start >= $seq->start() && $stop <= $seq->end()) || 
		        ($start <= $seq->end() && $stop >= $seq->end()) ||  
		        ($start <= $seq->start && $stop >= $seq->start()) ) {
			push(@arry,sprintf("%s:%s-%d-%d:%s-%d-%d",$seq->name(),$one->name(),$start,$stop,$model->name,$seq->start(),$seq->end()));
			$count++;
		    }
		}
	    }
	}
	if( $report ) {
	    print $report sprintf("Done Model %-25s - Found %d overlaps\n",$model->name(),$count);
	}

	$model = undef; # free Model
    } # end of for each family

    return @arry;
}


sub get_current_model {
    my $self = shift;
    my $name = shift;

    $self->name($name);
    $self->get_file_based_model("$current_dir/$name");
}


sub get_file_based_model {
    my $self = shift;
    my ($file);
    my $dir = shift;

    $self->name($dir);
    $file = "$dir/DESC";
    $self->getpara_file($file);
    if( !(-e $file) ) {
	carp("Could not get model DESC file for $name");
	return undef;
    }

    $file = "$dir/HMM";
    $self->hmmfile("$file");
    if( !(-e $file) ) {
	carp("Could not get model HMM file for $name");
	return undef;
    }


    $file = "$dir/random";

    if( -e $file  ) {
	$file = sprintf("$dir/random");
	$self->rndfile($file);
    }

    if( !open(ALIGN_IN,"$dir/SEED") ) {
	carp("Could not open SEED alignment for $name. Bad news! $!");
	return undef;
    }
    
    $seed = new Align;
    $seed->read_Pfam(\*ALIGN_IN);
    close(ALIGN_IN);
    $self->seed($seed);

    if( !open(ALIGN_IN,"$dir/ALIGN") ) {
	carp("Could not open ALIGN alignment for $name. Bad news! $!");
	return undef;
    }

    $align = new Align;
    $align->read_Pfam(\*ALIGN_IN);
    $self->align($align);
    close(ALIGN_IN);


    return $self;
}
    
sub getstd_model {
    my $self = shift;
    my $file;
    my $name = $self->name();

    if( !$self->name() ) {
	carp("Attempting to populate a PfamModel with no name! Impossible");
	return 0;
    }

    $file = sprintf("$release_dir/%s.html",$self->name());
    $self->getpara_file($file);
    if( !(-e $file) ) {
	carp("Could not get model html file for $name");
    }

    $file = sprintf("$release_dir/%s.HMM",$self->name());
    $self->hmmfile("$file");

    if( $self->random() == 1 ) {
	$file = sprintf("$release_dir/%s.random",$self->name());
	$self->rndfile($file);
    }

    if( !(-e $file) ) {
	carp("Could not get model HMM file for $name");
    }


    return $self;
}

    
sub getpara_file {
    my $self = shift;
    my $file = shift;
    my $prog,$level;
    my $random = 0;

    if( !open(IN,"$file") ) {
	carp("Could not open $file for get parameters");
	return $self;
    }

   
    while(<IN>) {
	if( !$self->name() && /^ID\s+(\S+)/ ) {
	    $name = $1;
	    $self->name($name);
	}

	if( /^GA.*(hmm[lf]s).*([0-9]*)/ ) {
	    $prog = $1;
	    $level = $2;
	    if( /hmm[lf]s[^\-]*\-r/ ) {
		$random = 1;
	    }
	    last;
	}

	if( /^Gathering method:.*(hmm[lf]s).+([0-9]*)/ ) {
	    $prog = $1;
	    $level = $2;
	    if( /hmm[lf]s[^\-]*\-r/ ) {
		$random = 1;
	    }
	    last;
	}
    }
    close(IN);

    if( $level < 15 ) {
	$level = 15;
    }

    $self->hmmprog($prog);
    $self->thresh($level); 
    $self->random($random);
    
    return $self;
}

 
sub new {
    my $ref = shift;
    my $class = ref($ref) || $ref;
    
    my $self = {
	_permitted => \%fields,
	%fields, };

    $self->{domain} = [];

    bless $self, $class;
    return $self;
}



sub AUTOLOAD {
    my $self = shift;
    my $type = ref($self) || warn "Ok, $self is not an object of mine!";
    my $name = $AUTOLOAD;
    $name =~ s/.*://; #get only the bit we want
    unless (exists $self->{_permitted}->{$name} ) {
	warn "In type $type, can't access $name $keys  ";
    }
    if (@_) {
	return $self->{$name} = shift;
    } else {
	return $self->{$name}
    }
}

    
sub DESTROY {  }

END { }       # module clean-up code here (global destructor)


