

package PrositePattern;

use DomainSet; 
use vars qw($AUTOLOAD);


#
# trues is a @ array of NAME:accession strings
#

my %fields = (
	      name  => undef,
	      pattern => undef,
	      regex   => undef,
	      accession => undef,
	      trues_pat => undef,
	      
);

sub new {
    my $ref = shift;
    my $class = ref($ref) || $ref;
    
    my $self = {
	_permitted => \%fields,
	%fields, };
    bless $self, $class;
    return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $type = ref($self) || warn "Ok, $self is not an object of mine!";
    my $name = $AUTOLOAD;
    $name =~ s/.*://; #get only the bit we want
    unless (exists $self->{_permitted}->{$name} ) {
	warn "In type $type, can't access $name $keys  ";
    }
    if (@_) {
	return $self->{$name} = shift;
    } else {
	return $self->{$name}
    }
}


sub get_prosite_matches {
    my $self = shift;
    my $seq  = shift;
    my $dom;
    my $pattern;
    my $ds;

    $ds = new DomainSet;
    $pattern = $self->regex();

    #
    # ok, loop over sequence to get out patterns
    # m/xxx/g in scalar context returns true while we have guys left.
    # we can get out the start point with pos(). We have the string 
    # that matched, hence we have the she-bang ;)
    #

    print "Pattern is $pattern\n";

    while( $seq =~ m/($pattern)/g ) {


	$match = $1;
	$start = pos($seq); # gets position of string
	$end   = $start + length($match);

	print "Got one -$match- $start\n";
	
	$dom = new Domain;
	$dom->start($start);
	$dom->end($end);
	$dom->seq($match);
	$dom->type('prosite-pat');

	$ds->add_Domain($dom);

	# $dom->show(\*STDOUT);
	
    }

    $ds->show(\*STDOUT);

    return $ds;
}


sub prosite_to_perl {
    my $pro = shift;

    $_ = $pro;

    s/-//g;                         # removes 'spacing' -
    s/\{/\[^/g;                     # {} means NOT [] hence [^ ]
    s/\}/\]/g;
    #                               [ ] is identical to the perl meaning
    s/\((\d+)\)/\{$1,$1\}/g;        # (3) is 3 of this residue
    s/\((\d+)\,(\d+)\)/\{$1,$2\}/g; # (6,12) is at least 6 no more than 12 of this
    s/x/\\w/g;                      # x means any aa - map to any char.
    s/\>/\$/;                       # < start of sequence
    s/\</\^/;                       # > end of sequence 
    s/\.//g;                        # end of pattern - remove ;)

    return $_;
}


sub read{
    my $self = shift;
    my $file = shift;
    my $pattern = "";

    
    while ( <$file> ) {
	if( /^ID\s+(\S+);/ ) { 
	    $temp = $1;
	    $self->name($temp);
	} elsif ( /^AC\s+(\S+);/ ) {
	    $temp = $1;
	    $self->accession($temp);
	} elsif ( /^PA\s+(\S+)/ ) {
	    $temp = $1;
	    $pattern .= $temp;
	} elsif ( /^DR/ ) {
	    s/^DR//;
	    chop;
	    @set = split(/;/);
	    foreach $seq (@set) {
		if( !/^\s+(\S+)\s*,\s+(\S+)\s*,\s+(\w)/ ) {
		    warn("Got a bad prosite entry seq [$seq]");
		    next;
		}
		$acc = $1;
		$name = $2;
		$c = $3;

		if( $c eq "T" ) {
		    push(@{$self->{trues_pat}},"$name:$acc");
		}
	    }
	}
    }

    $self->pattern($pattern);
    $self->regex(&prosite_to_perl($pattern));

}


sub show {
    my $self = shift;
    my $file = shift;
    my $name;
    my $acc;
    my $pat;
    my @arr;
    my $true;
    my $reg;
    
    $name = $self->name();
    $acc  = $self->accession();
    $pat  = $self->pattern();
    $reg  = $self->regex();

    print $file "ID   $name\n";
    print $file "AC   $acc\n";
    print $file "PA   $pat\n";
    print $file "PE   $reg\n";

    @arr = @{$self->trues_pat()};
    foreach $true (@arr) {
	print $file "True $true\n";
    }
}














