## Bioperl Test Harness Script for Modules

# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl test.t'

#-----------------------------------------------------------------------
## perl test harness expects the following output syntax only!
## 1..3
## ok 1  [not ok 1 (if test fails)]
## 2..3
## ok 2  [not ok 2 (if test fails)]
## 3..3
## ok 3  [not ok 3 (if test fails)]
##
## etc. etc. etc. (continue on for each tested function in the .t file)
#-----------------------------------------------------------------------


## We start with some black magic to print on failure.
BEGIN { $| = 1; print "1..13\n"; }
END {print "not ok 1\n" unless $loaded;}


use Bio::Pfam;

$loaded = 1;
print "ok 1\n";    # 1st test passes.


## End of black magic.
##
## Insert additional test code below but remember to change
## the print "1..x\n" in the BEGIN block to reflect the
## total number of tests that will be run. 

$db = &Bio::Pfam::latest_release();

if( !defined $db ) { print "not ok 2\n"; }
else { print "ok 2\n"; }

open (GETZ, "getz \'[pfam-acc:PF00001]\' |");
my $var = <GETZ>;

if( !defined $var ) { print "not ok 3\n"; }
else { print "ok 3\n"; }

undef $var;
open (GETZ, "getz \'[pfama-acc:PF00001]\' |");
$var = <GETZ>;

if( !defined $var ) { print "not ok 4\n"; }
else { print "ok 4\n"; }

undef $var;
open (GETZ, "getz \'[pfamanew-acc:PF00001]\' |");
$var = <GETZ>;

if( !defined $var ) { print "not ok 5\n"; }
else { print "ok 5\n"; }

undef $var;
open (GETZ, "getz \'[pfamnew-acc:PF00001]\' |");
$var = <GETZ>;

if( !defined $var ) { print "not ok 6\n"; }
else { print "ok 6\n"; }

undef $var;
open (GETZ, "getz \'[pfamb-acc:PB000001]\' |");
$var = <GETZ>;

if( !defined $var ) { print "not ok 7\n"; }
else { print "ok 7\n"; }

undef $var;
open (GETZ, "getz \'[pfambnew-acc:PB000001]\' |");
$var = <GETZ>;

if( !defined $var ) { print "not ok 8\n"; }
else { print "ok 8\n"; }

undef $var;
open (GETZ, "getz \'[swisspfam-acc:P35520]\' |");
$var = <GETZ>;

if( !defined $var ) { print "not ok 9\n"; }
else { print "ok 9\n"; }

undef $var;
open (GETZ, "getz \'[swisspfamnew-acc:P35520]\' |");
$var = <GETZ>;

if( !defined $var ) { print "not ok 10\n"; }
else { print "ok 10\n"; }

undef $var;
open (GETZ, "getz \'[pfamseq-acc:P35520]\' |");
$var = <GETZ>;

if( !defined $var ) { print "not ok 11\n"; }
else { print "ok 11\n"; }

###########
# TEST 12 # Test Pfam-A
###########

# Try and test if getz is returning correct result with respect to website
my $i;
my $acc="PF00001";
open (GETZ, "getz \'[pfama-acc:$acc]\' -e |");
while (<GETZ>){
  if (/^\S+\/\d+-\d+\s+\S+/){ # Matching alignment only
    $i++;
  }
}
close (GETZ);

my ($db) = &Bio::Pfam::latest_release();
my $entry = $db->get_EntryA_by_acc("$acc");
my $full = $entry->full();
my $j = my @foo = $full->eachSeq;

# print STDERR "getz finds $i and website finds $j in $acc\n";

if($i  != $j) { print "not ok 12\n";}
else { print "ok 12\n"; }

###########
# TEST 13 # Test Pfam-B
###########

$i=0;
my $pfamb_acc="PB000100";
open (GETZ, "getz \'[pfamb-acc:$pfamb_acc]\' -e |");
while (<GETZ>){
  if (/^\S+\/\d+-\d+\s+\S+/){ # Matching alignment only
    $i++;
  }
}
close (GETZ);

$entry = $db->get_EntryB_by_acc("$pfamb_acc");
$full = $entry->full();
$j = @foo = $full->eachSeq;

# print STDERR "getz finds $i and website finds $j in $pfamb_acc\n";

if($i  != $j) { print "not ok 13\n";}
else { print "ok 13\n"; }



