## Bioperl Test Harness Script for Modules

# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl test.t'

#-----------------------------------------------------------------------
## perl test harness expects the following output syntax only!
## 1..3
## ok 1  [not ok 1 (if test fails)]
## 2..3
## ok 2  [not ok 2 (if test fails)]
## 3..3
## ok 3  [not ok 3 (if test fails)]
##
## etc. etc. etc. (continue on for each tested function in the .t file)
#-----------------------------------------------------------------------


## We start with some black magic to print on failure.
BEGIN { $| = 1; print "1..6\n"; }
END {print "not ok 1\n" unless $loaded;}


use Bio::Pfam;

$loaded = 1;
print "ok 1\n";    # 1st test passes.


## End of black magic.
##
## Insert additional test code below but remember to change
## the print "1..x\n" in the BEGIN block to reflect the
## total number of tests that will be run. 

$db = &Bio::Pfam::default_db();

if( !defined $db ) { print "not ok 2\n"; }
else { print "ok 2\n"; }

$id = $db->acc2id('PF00076');

if( !defined $id ) { print "not ok 3\n"; }
else { print "ok 3\n"; }

$acc  = $db->id2acc($id);

if( !defined $id ) { print "not ok 4\n"; }
else { print "ok 4\n"; }

if( $acc eq 'PF00076' ) { print "ok 5\n" }
else { print STDERR "Accession number is $acc, not PF00076\n"; print "not ok 5\n"; }

##########
# Test 6 #
##########

$db = &Bio::Pfam::default_db();
$error_total=0;
foreach $acc ( $db->get_allacc('DATE') ) {
  $id="";
  $entry = $db->get_EntryA_by_acc($acc);
  $id=$entry->id();
  if (! $id){
      $error_seen=$acc;
      $error_total++;
  } 
}

if ($error_seen){
    print "not ok 6\n";
    print STDERR "\n\nNo id for $error_seen! There are $error_total missing id's.\n\n";
} else {print "ok 6\n";}

########## Test that we can get id's for all entries!
# Test 7 # for latest release.
##########

#$db = &Bio::Pfam::latest_release();
#$error_total=0;
#foreach $acc ( $db->get_allacc('DATE') ) {
#  $id="";
#  $entry = $db->get_EntryA_by_acc($acc);
#  $id=$entry->id();
#  if (! $id){
#      $error_seen=$acc;
#      $error_total++;
#  } 
#}

#if ($error_seen){
#    print "not ok 7\n";
#    print STDERR "\n\nNo id for $error_seen! There are $error_total missing id's.\n\n";
#} else {print "ok 7\n";}




